@@ -93,6 +93,9 @@ class Geoanalysis:
                 & (self.df_analysis["Crop"] == self.crop)
                 & (self.df_analysis["Model"] == self.model)
             ]
+
+            # Drop columns that are empty
+            # self.df_analysis = self.df_analysis.dropna(axis=1, how="all")
         except Exception as e:
             pass
 
@@ -162,9 +165,9 @@ class Geoanalysis:
             return pd.DataFrame(), pd.DataFrame()
 
         df_metrics = self._compute_metrics(df)
-        # df_metrics = self._process_metrics(df_metrics)
-        # self._plot_metrics(df_metrics)
+        df_metrics = self._process_metrics(df_metrics)
 
+        self._plot_metrics(df_metrics)
         df_regional_metrics_by_year = self._compute_regional_metrics(
             df, by="Harvest Year"
         )
@@ -172,11 +175,13 @@ class Geoanalysis:
             df_regional_metrics_by_year
         )
         df_regional_metrics = self._average_mape(df_regional_metrics_by_year)
-        breakpoint()
-        self._store_results(None, df_regional_metrics, df_regional_metrics_by_year)
 
+        self._store_results(
+            df_metrics, df_regional_metrics, df_regional_metrics_by_year
+        )
         df_national_yield = self._compute_national_yield(df)
         self._plot_national_yield(df_national_yield)
+        self._plot_regional_yield_scatter(df)
 
         return df_metrics, df_regional_metrics, df_national_yield
 
@@ -193,12 +198,13 @@ class Geoanalysis:
             .apply(self.annual_metrics)
             .reset_index()
         )
-        breakpoint()
-        return df_metrics.pivot_table(
-            index=["Country", "Model", "Harvest Year", "Stage Name", "Stage Range"],
-            columns="level_5",
-            values=0,
-        ).reset_index()
+
+        # return df_metrics.pivot_table(
+        #    index=["Country", "Model", "Harvest Year", "Stage Name", "Stage Range"],
+        #    columns="level_5",
+        #    values=0,
+        # ).reset_index()
+        return df_metrics
 
     def _process_metrics(self, df_metrics):
         # Assign each unique Stage Name a unique integer identifier
@@ -241,12 +247,15 @@ class Geoanalysis:
         else:
             return df.groupby(cols).apply(self.regional_metrics).reset_index()
 
-    def _select_top_years(self, df_regional_metrics):
-        return (
-            df_regional_metrics.groupby(["Country", "Region"])
-            .apply(lambda x: self.select_top_N_years(x, 10))
-            .reset_index(drop=True)
-        )
+    def _select_top_years(self, df_regional_metrics, top_N=-1):
+        if top_N == -1:
+            return df_regional_metrics
+        else:
+            return (
+                df_regional_metrics.groupby(["Country", "Region"])
+                .apply(lambda x: self.select_top_N_years(x, 10))
+                .reset_index(drop=True)
+            )
 
     def _average_mape(self, df_regional_metrics):
         cols = [
@@ -329,18 +338,31 @@ class Geoanalysis:
 
             con.commit()
 
-    def _compute_national_yield(self, df):
-        # Compute observed and predicted national yield by multiplying Yield (tn per ha) by Area (ha)
+    def _compute_national_yield(self, df_region):
+        # Define column names
         observed = "Observed Yield (tn per ha)"
         predicted = "Predicted Yield (tn per ha)"
         area_ha = "Area (ha)"
 
-        df.loc[:, observed] = df[observed] * df[area_ha]
-        df.loc[:, predicted] = df[predicted] * df[area_ha]
+        df_tmp = df_region.copy()
+
+        # Fill
+        df_tmp[area_ha] = df_tmp.groupby("Country")[area_ha].transform(
+            lambda x: x.fillna(x.median())
+        )
+
+        # Log that we are filling missing values with the median
+        self.logger.info(
+            f"Filling missing values in {area_ha} with the median for each country"
+        )
+
+        # Compute observed and predicted national yield by multiplying Yield (tn per ha) by Area (ha)
+        df_tmp[observed] = df_tmp[observed] * df_tmp[area_ha]
+        df_tmp[predicted] = df_tmp[predicted] * df_tmp[area_ha]
 
         # Group by Country and Harvest Year, then sum the National Yield and Area
         df_national_yield = (
-            df.groupby(["Country", "Harvest Year"])
+            df_tmp.groupby(["Country", "Harvest Year"])
             .agg({observed: "sum", predicted: "sum", area_ha: "sum"})
             .reset_index()
         )
@@ -355,53 +377,59 @@ class Geoanalysis:
 
         return df_national_yield
 
-    def _plot_national_yield(self, df_national_yield, use_different_colors=True):
+    def _plot_regional_yield_scatter(self, df):
+        """
+        Plot observed vs predicted yield for all regions and all years.
+        """
         from sklearn.metrics import (
             mean_squared_error,
             r2_score,
             mean_absolute_percentage_error,
         )
 
-        x = df_national_yield["Harvest Year"]
-        y_observed = df_national_yield["Observed Yield (tn per ha)"]
-        y_predicted = df_national_yield["Predicted Yield (tn per ha)"]
+        # Ensure 'Harvest Year' is numeric
+        df.loc[:, "Harvest Year"] = pd.to_numeric(df["Harvest Year"], errors="coerce")
 
+        # Extract data
+        y_observed = df["Observed Yield (tn per ha)"]
+        y_predicted = df["Predicted Yield (tn per ha)"]
+        years = df["Harvest Year"]
+
+        # Generate colors for years
+        cmap = plt.cm.viridis  # Colormap for years
+        norm = plt.Normalize(
+            vmin=years.min(), vmax=years.max()
+        )  # Normalize years to colormap
+        colors = [cmap(norm(year)) for year in years]
+
+        # Create the plot
         with plt.style.context("science"):
-            plt.figure(figsize=(10, 6))
+            fig, ax = plt.subplots(figsize=(10, 6))
 
-            import palettable as pal
+            # Add gridlines
+            ax.grid(True, linestyle="--", alpha=0.5)
 
-            colors = pal.tableau.Tableau_20.mpl_colors
-            colors = colors[: len(x)]
+            # Scatter plot with colors representing years
+            scatter = ax.scatter(y_observed, y_predicted, color=colors, s=50)
 
-            # Add dashed gray grid lines with alpha=0.5
-            plt.grid(True, linestyle="--", alpha=0.5)
-
-            for i in range(len(x)):
-                plt.scatter(y_observed[i], y_predicted[i], color=colors[i], label=x[i])
-
-            # X and Y-axis range from 0 to the maximum observed/predicted yield * 1.1
+            # Add 1:1 diagonal line
             max_yield = max(y_observed.max(), y_predicted.max()) * 1.25
-            plt.xlim(0, max_yield)
-            plt.ylim(0, max_yield)
+            ax.plot([0, max_yield], [0, max_yield], color="gray", linestyle="--")
 
-            # Add a line diagonally representing 1:1
-            plt.plot([0, max_yield], [0, max_yield], color="gray", linestyle="--")
-
-            # Calculate metrics
+            # Calculate and display metrics
             rmse = np.sqrt(mean_squared_error(y_observed, y_predicted))
             mape = mean_absolute_percentage_error(y_observed, y_predicted)
             r2 = r2_score(y_observed, y_predicted)
+            n_points = len(y_observed)  # Number of data points
 
-            # Annotate metrics
-            textstr = "\n".join(
-                (
-                    f"RMSE: {rmse:.2f} tn/ha",
-                    f"MAPE: {mape:.2%}",
-                    f"R²: {r2:.2f}",
-                )
+            textstr = (
+                f"RMSE: {rmse:.2f} tn/ha\n"
+                f"MAPE: {mape:.2%}\n"
+                f"$r^2$: {r2:.2f}\n"
+                f"N: {n_points}"
             )
-            plt.gca().annotate(
+
+            ax.annotate(
                 textstr,
                 xy=(0.05, 0.95),
                 xycoords="axes fraction",
@@ -409,17 +437,114 @@ class Geoanalysis:
                 verticalalignment="top",
             )
 
-            plt.xlabel("Observed Yield (tn/ha)")
-            plt.ylabel("Predicted Yield (tn/ha)")
-            # Place legend outside the plot to the right without a border
-            plt.legend(
-                title="Year",
-                bbox_to_anchor=(1.05, 1),
-                loc="upper left",
-                edgecolor="none",
-            )
+            # Set axis limits and labels
+            ax.set_xlabel("Observed Yield (tn/ha)")
+            ax.set_ylabel("Predicted Yield (tn/ha)")
+            ax.set_xlim(0, max_yield)
+            ax.set_ylim(0, max_yield)
+
+            # Add colorbar for years
+            sm = plt.cm.ScalarMappable(cmap=cmap, norm=norm)
+            sm.set_array([])
+            cbar = fig.colorbar(sm, ax=ax, aspect=50, pad=0.02)
+            cbar.set_label("Harvest Year")
+
+            # Set equispaced ticks for exactly 5 points
+            ticks = np.linspace(
+                years.min(), years.max(), 5, dtype=int
+            )  # 5 equispaced ticks
+            cbar.set_ticks(ticks)
+            cbar.ax.set_yticklabels([str(tick) for tick in ticks])
+
             plt.tight_layout()
 
+            # Save the plot
+            fname = f"scatter_all_regions_{self.country}_{self.crop}.png"
+            plt.savefig(self.dir_analysis / fname, dpi=250)
+            plt.close()
+
+    def _plot_national_yield(self, df_national_yield):
+        from sklearn.metrics import (
+            mean_squared_error,
+            r2_score,
+            mean_absolute_percentage_error,
+        )
+
+        # Ensure 'Harvest Year' is numeric
+        df_national_yield["Harvest Year"] = pd.to_numeric(
+            df_national_yield["Harvest Year"], errors="coerce"
+        )
+
+        # Extract data
+        x = df_national_yield["Harvest Year"]
+        y_observed = df_national_yield["Observed Yield (tn per ha)"]
+        y_predicted = df_national_yield["Predicted Yield (tn per ha)"]
+
+        # Generate colors for years
+        cmap = plt.cm.viridis  # Colormap for years
+        norm = plt.Normalize(vmin=x.min(), vmax=x.max())  # Normalize years to colormap
+        colors = [cmap(norm(year)) for year in x]
+
+        # Create the plot
+        with plt.style.context("science"):
+            fig, ax = plt.subplots(figsize=(10, 6))  # Explicitly define axes
+
+            max_yield = max(y_observed.max(), y_predicted.max()) * 1.25
+
+            # Add gridlines
+            ax.grid(True, linestyle="--", alpha=0.5)
+
+            # Scatter plot with uniform size and dynamic colors
+            for year, obs, pred, color in zip(x, y_observed, y_predicted, colors):
+                ax.scatter(obs, pred, color=color, s=50, label=year)
+
+            # Add 1:1 diagonal line
+            ax.plot([0, max_yield], [0, max_yield], color="gray", linestyle="--")
+
+            # Calculate and display metrics
+            rmse = np.sqrt(mean_squared_error(y_observed, y_predicted))
+            mape = mean_absolute_percentage_error(y_observed, y_predicted)
+            r2 = r2_score(y_observed, y_predicted)
+
+            n_points = len(y_observed)  # Number of data points
+
+            textstr = (
+                f"RMSE: {rmse:.2f} tn/ha\n"
+                f"MAPE: {mape:.2%}\n"
+                f"$r^2$: {r2:.2f}\n"
+                f"N: {n_points}"
+            )
+
+            ax.annotate(
+                textstr,
+                xy=(0.05, 0.95),
+                xycoords="axes fraction",
+                fontsize=12,
+                verticalalignment="top",
+            )
+
+            # Set axis limits and labels
+            ax.set_xlabel("Observed Yield (tn/ha)")
+            ax.set_ylabel("Predicted Yield (tn/ha)")
+            ax.set_xlim(0, max_yield)
+            ax.set_ylim(0, max_yield)
+
+            # Add legend for years
+            sm = plt.cm.ScalarMappable(cmap=cmap, norm=norm)
+            sm.set_array([])
+            cbar = fig.colorbar(
+                sm, ax=ax, aspect=50, pad=0.02
+            )  # Specify the axis explicitly
+            cbar.set_label("Harvest Year")
+
+            # Set equispaced ticks for exactly 5 points
+            ticks = np.linspace(x.min(), x.max(), 5, dtype=int)  # 5 equispaced ticks
+            cbar.set_ticks(ticks)
+            cbar.ax.set_yticklabels([str(tick) for tick in ticks])
+
+            plt.tight_layout()
+
+            # Save the plot
             fname = f"scatter_{self.country}_{self.crop}.png"
             plt.savefig(self.dir_analysis / fname, dpi=250)
             plt.close()
@@ -431,34 +556,50 @@ class Geoanalysis:
         country = self.country.title().replace("_", " ")
         crop = self.crop.title().replace("_", " ")
         file = dir_statistics / f"{country}_{crop}_statistics_s1_{self.method}.csv"
-        df_historic = pd.read_csv(file)
+        df_all = pd.read_csv(file)
 
-        df_historic = df_historic[["Region", "Harvest Year", "Yield (tn per ha)"]]
+        # Keep only the relevant columns and drop NaNs
+        df_all = df_all[["Region", "Harvest Year", "Yield (tn per ha)"]].dropna()
 
-        # Drop rows with NaN values
-        df_historic = df_historic.dropna()
+        # --- For computing the % of total production ---
+        # Determine unique years and sort them (in case they aren't already)
+        years = sorted(df_all["Harvest Year"].unique())
+        # Subset dataframe to include only the last 5 years of the dataset
+        last_five_years = years[-5:]
+        df_recent = df_all[df_all["Harvest Year"].isin(last_five_years)]
 
-        # Determine unique years
-        years = df_historic["Harvest Year"].unique()
-
-        # Subset dataframe to only include the last years of the dataset
-        df_historic = df_historic[df_historic["Harvest Year"].isin(years[-5:])]
-
-        # For each region, compute the % of the total production
-        df_historic = (
-            df_historic.groupby("Region")["Yield (tn per ha)"]
+        # For each region, compute the % of total production (using yield sum over the last five years)
+        df_pct = (
+            df_recent.groupby("Region")["Yield (tn per ha)"]
             .sum()
             .pipe(lambda x: x / x.sum() * 100)
             .to_frame(name="% of total Area (ha)")
             .reset_index()
         )
-        # Find median yield for each region
-        # df_historic = (
-        #     df_historic.groupby("Region")["Yield (tn per ha)"]
-        #     .median()
-        #     .to_frame(name="Median Yield (tn per ha)")
-        #     .reset_index()
-        # )
+
+        # --- For computing median yields ---
+        # Compute median yield for 2018 - 2022
+        df_median_2018_2022 = (
+            df_all[df_all["Harvest Year"].between(2018, 2022)]
+            .groupby("Region")["Yield (tn per ha)"]
+            .mean()
+            .rename(f"Median Yield (tn per ha) (2018-2022)")
+            .reset_index()
+        )
+
+        # Compute median yield for 2013 - 2017
+        df_median_2013_2017 = (
+            df_all[df_all["Harvest Year"].between(2013, 2017)]
+            .groupby("Region")["Yield (tn per ha)"]
+            .mean()
+            .rename("Median Yield (tn per ha) (2013-2017)")
+            .reset_index()
+        )
+
+        # Merge the median yield columns with the % of total production dataframe
+        df_historic = df_pct.merge(df_median_2018_2022, on="Region", how="left").merge(
+            df_median_2013_2017, on="Region", how="left"
+        )
 
         return df_historic
 
@@ -486,7 +627,7 @@ class Geoanalysis:
         self.df_analysis["Anomaly"] = (
             self.df_analysis[self.predicted]
             * 100.0
-            / self.df_analysis["Median Yield (tn per ha)"]
+            / self.df_analysis["Median Yield (tn per ha) (2018-2022)_y"]
         )
 
         # Compute the yield from the last year
@@ -526,7 +667,11 @@ class Geoanalysis:
                 "ADM1_NAME" if self.admin_zone == "admin_1" else "ADM2_NAME"
             )
             analysis_years = df_model["Harvest Year"].unique()
-            for idx, year in enumerate(tqdm(analysis_years, desc="Map")):
+            pbar = tqdm(analysis_years, leave=False)
+            for idx, year in enumerate(pbar):
+                pbar.set_description(f"Map {year}")
+                pbar.update()
+
                 df_harvest_year = df_model[df_model["Harvest Year"] == year]
 
                 for time_period in tqdm(
@@ -538,7 +683,7 @@ class Geoanalysis:
                     #
                     #                 """ % of total area """
                     if idx == 0:
-                        fname = f"{self.country}_{self.crop}_perc_area.png"
+                        fname = f"map_{self.country}_{self.crop}_perc_area.png"
                         col = "% of total Area (ha)"
                         plot.plot_df_shpfile(
                             self.dg,  # dataframe containing adm1 name and polygon
@@ -554,13 +699,13 @@ class Geoanalysis:
                             cmap=pal.scientific.sequential.Bamako_20_r,
                             series="sequential",
                             show_bg=False,
-                            annotate_regions=False,
+                            annotate_regions=self.annotate_regions,
                             annotate_region_column=annotate_region_column,
                             loc_legend="lower left",
                         )
                     #
-                    #                 #     """ Unique regions """
-                    fname = f"{self.country}_{self.crop}_region_ID.png"
+                    """ Unique regions """
+                    fname = f"map_{self.country}_{self.crop}_region_ID.png"
                     col = "Region_ID"
                     df_model[col] = df_model[col].astype(int) + 1
                     if len(df_model["Region_ID"].unique() > 1):
@@ -569,6 +714,7 @@ class Geoanalysis:
                             int(key): key
                             for key in df_time_period["Region_ID"].unique()
                         }
+
                         plot.plot_df_shpfile(
                             self.dg,  # dataframe containing adm1 name and polygon
                             df_model,  # dataframe containing information that will be mapped
@@ -586,7 +732,7 @@ class Geoanalysis:
                             show_bg=False,
                             alpha_feature=1,
                             use_key=True,
-                            annotate_regions=False,
+                            annotate_regions=self.annotate_regions,
                             annotate_region_column=annotate_region_column,
                             loc_legend="lower left",
                         )
@@ -610,13 +756,13 @@ class Geoanalysis:
                     #     cmap=pal.cartocolors.diverging.Geyser_5_r,
                     #     series="sequential",
                     #     show_bg=False,
-                    #     annotate_regions=False,
+                    #     annotate_regions=True,
                     #     annotate_region_column=annotate_region_column,
                     #     loc_legend="lower left",
                     # )
 
                     """ Predicted Yield """
-                    fname = f"{fname_prefix}_{self.crop}_{time_period}_{year}_predicted_yield.png"
+                    fname = f"map_{fname_prefix}_{self.crop}_{time_period}_{year}_predicted_yield.png"
                     plot.plot_df_shpfile(
                         self.dg,  # dataframe containing adm1 name and polygon
                         df_harvest_year,  # dataframe containing information that will be mapped
@@ -631,11 +777,78 @@ class Geoanalysis:
                         cmap=pal.scientific.sequential.Bamako_20_r,
                         series="sequential",
                         show_bg=False,
-                        annotate_regions=False,
+                        annotate_regions=self.annotate_regions,
                         annotate_region_column=annotate_region_column,
                         loc_legend="lower left",
                     )
 
+                    # Make map of predicted yield by country
+                    for country in countries:
+                        df_country = df_model[df_model["Country"] == country.lower().replace(" ", "_")]
+                        fname = f"map_perc_area_{self.country}_{self.crop}.png"
+                        col = "% of total Area (ha)"
+                        plot.plot_df_shpfile(
+                            self.dg,  # dataframe containing adm1 name and polygon
+                            df_country,  # dataframe containing information that will be mapped
+                            merge_col="Country Region",  # Column on which to merge
+                            name_country=[country],  # Plot global map
+                            name_col=col,  # Which column to plot
+                            dir_out=self.dir_plot / str(year),  # Output directory
+                            fname=fname,  # Output file name
+                            label=f"% of Total Area (ha)\n{self.crop.title()}",
+                            vmin=df_country[col].min(),
+                            vmax=df_country[col].max(),
+                            cmap=pal.scientific.sequential.Bamako_20_r,
+                            series="sequential",
+                            show_bg=False,
+                            annotate_regions=self.annotate_regions,
+                            annotate_region_column=annotate_region_column,
+                            loc_legend="lower left",
+                        )
+
+                        df_country = df_harvest_year[df_harvest_year["Country"] == country.lower().replace(" ", "_")]
+                        fname = f"map_predicted_yield_{country}_{self.crop}_{time_period}_{year}.png"
+                        plot.plot_df_shpfile(
+                            self.dg,  # dataframe containing adm1 name and polygon
+                            df_country,  # dataframe containing information that will be mapped
+                            merge_col="Country Region",  # Column on which to merge
+                            name_country=[country],  # Plot global map
+                            name_col="Predicted Yield (tn per ha)",  # Which column to plot
+                            dir_out=self.dir_plot / str(year),  # Output directory
+                            fname=fname,  # Output file name
+                            label=f"Predicted Yield (Mg/ha)\n{self.crop.title()}, {year}",
+                            vmin=df_country[self.predicted].min(),
+                            vmax=df_country[self.predicted].max(),
+                            cmap=pal.scientific.sequential.Bamako_20_r,
+                            series="sequential",
+                            show_bg=False,
+                            annotate_regions=self.annotate_regions,
+                            annotate_region_column=annotate_region_column,
+                            loc_legend="lower left",
+                        )
+
+                        fname = (
+                            f"map_anomaly_{country}_{self.crop}_{time_period}_{year}.png"
+                        )
+                        plot.plot_df_shpfile(
+                            self.dg,  # dataframe containing adm1 name and polygon
+                            df_country,  # dataframe containing information that will be mapped
+                            merge_col="Country Region",  # Column on which to merge
+                            name_country=[country],  # Plot global map
+                            name_col="Anomaly",  # Which column to plot
+                            dir_out=self.dir_plot / str(year),  # Output directory
+                            fname=fname,  # Output file name
+                            label=f"% of {self.number_lag_years}-year Median Yield\n{self.crop.title()}, {year}",
+                            vmin=df_country["Anomaly"].min(),
+                            vmax=110,  # df_harvest_year["Anomaly"].max(),
+                            cmap=pal.cartocolors.diverging.Geyser_5_r,
+                            series="sequential",
+                            show_bg=False,
+                            annotate_regions=self.annotate_regions,
+                            annotate_region_column=annotate_region_column,
+                            loc_legend="lower left",
+                        )
+
                     """ Ratio of Predicted to last Year Yield """
                     # fname = f"{self.country}_{self.crop}_{time_period}_{year}_ratio_last_year_yield.png"
                     # plot.plot_df_shpfile(
@@ -652,32 +865,32 @@ class Geoanalysis:
                     #     cmap=pal.scientific.sequential.Bamako_20_r,
                     #     series="sequential",
                     #     show_bg=False,
-                    #     annotate_regions=False,
+                    #     annotate_regions=True,
                     #     annotate_region_column=annotate_region_column,
                     #     loc_legend="lower left",
                     # )
 
                     # Area
                     # breakpoint()
-                    # if df_time_period["Area (ha)"].notna().all():
-                    #     fname = f"{self.country}_{self.crop}_{year}_area.png"
-                    #     plot.plot_df_shpfile(
-                    #         self.dg,  # dataframe containing adm1 name and polygon
-                    #         df_time_period,  # dataframe containing information that will be mapped
-                    #         merge_col="Country Region",  # Column on which to merge
-                    #         name_country=country,  # Plot global map
-                    #         name_col="Area (ha)",  # Which column to plot
-                    #         dir_out=self.plot_dir / str(year),  # Output directory
-                    #         fname=fname,  # Output file name
-                    #         label=f"{self.predicted}\n{self.crop.title()}, {time_period}",
-                    #         vmin=df_time_period[self.predicted].min(),
-                    #         vmax=df_time_period[self.predicted].max(),
-                    #         cmap=pal.scientific.sequential.Bamako_20_r,
-                    #         series="sequential",
-                    #         show_bg=False,
-                    #         annotate_regions=True,
-                    #         loc_legend="lower left",
-                    #     )
+                    if df_time_period["Area (ha)"].notna().all():
+                        fname = f"map_{self.country}_{self.crop}_{year}_area.png"
+                        plot.plot_df_shpfile(
+                            self.dg,  # dataframe containing adm1 name and polygon
+                            df_time_period,  # dataframe containing information that will be mapped
+                            merge_col="Country Region",  # Column on which to merge
+                            name_country=countries,  # Plot global map
+                            name_col="Area (ha)",  # Which column to plot
+                            dir_out=self.dir_plot / str(year),  # Output directory
+                            fname=fname,  # Output file name
+                            label=f"Area (ha)\n{self.crop.title()}, {time_period}",
+                            vmin=df_time_period["Area (ha)"].min(),
+                            vmax=df_time_period["Area (ha)"].max(),
+                            cmap=pal.scientific.sequential.Bamako_20_r,
+                            series="sequential",
+                            show_bg=False,
+                            annotate_regions=self.annotate_regions,
+                            loc_legend="lower left",
+                        )
 
     def plot_metric(self, df, metric="$r^2$"):
         with plt.style.context("science"):
@@ -779,15 +992,22 @@ class Geoanalysis:
                         "name_shapefile": name_shapefile,
                     }
 
+        shp_file = self.parser.get(country, "boundary_file")
         self.dg = gpd.read_file(
-            self.dir_shapefiles / "adm_shapefile.shp",
+            self.dir_shapefiles / shp_file,
             engine="pyogrio",
         )
+        self.admin_col_name = self.parser.get(country, "admin_col_name")
+        self.annotate_regions = self.parser.getboolean(country, "annotate_regions")
 
-        # Hack rename Tanzania to United Republic of Tanzania
-        self.dg["ADMIN0"] = self.dg["ADMIN0"].replace(
-            "Tanzania", "United Republic of Tanzania"
-        )
+        # If ADMIN0 or ADM0_NAME is not in the shapefile, then add ADM0_NAME
+        if "ADMIN0" not in self.dg.columns and "ADM0_NAME" not in self.dg.columns:
+            self.dg.loc[:, "ADMIN0"] = country.title().replace("_", " ")
+
+        # If ADMIN1 or ADM1_NAME is not in the shapefile, then rename admin_col_name to ADMIN1
+        if "ADMIN1" not in self.dg.columns and "ADM1_NAME" not in self.dg.columns:
+            if admin_zone == "admin_1":
+                self.dg.rename(columns={self.admin_col_name: "ADMIN1"}, inplace=True)
 
         # Rename ADMIN0 to ADM0_NAME and ADMIN1 to ADM1_NAME and ADMIN2 to ADM2_NAME
         self.dg = self.dg.rename(
@@ -804,9 +1024,10 @@ class Geoanalysis:
         self.dg["Country Region"] = self.dg["Country Region"].str.cat(
             self.dg["ADM1_NAME"], sep=" "
         )
-        self.dg.loc[self.dg["ADM2_NAME"].notna(), "Country Region"] = (
-            self.dg["ADM0_NAME"] + " " + self.dg["ADM2_NAME"]
-        )
+        if "ADM2_NAME" in self.dg.columns:
+            self.dg.loc[self.dg["ADM2_NAME"].notna(), "Country Region"] = (
+                self.dg["ADM0_NAME"] + " " + self.dg["ADM2_NAME"]
+            )
         # Make it lower case
         self.dg["Country Region"] = (
             self.dg["Country Region"].str.lower().replace("_", " ")
@@ -830,28 +1051,39 @@ class RegionalMapper(Geoanalysis):
         self.read_data()
 
         self.clean_data()
-        self.plot_heatmap()
-        self.plot_kde()
-        self.plot_mape_map()
-        self.plot_mape_by_year()
+        if not self.df_regional.empty and self.df_regional_by_year.empty:
+            self.plot_heatmap()
+            self.plot_kde()
+            self.plot_mape_map()
+            self.plot_mape_by_year()
 
     def read_data(self):
         """Read data from the database."""
         con = sqlite3.connect(self.db_path)
 
         query = "SELECT * FROM regional_metrics"
-        self.df_regional = pd.read_sql_query(query, con)
+        try:
+            self.df_regional = pd.read_sql_query(query, con)
+        except:
+            self.logger.error("Failed to read data from regional_metrics")
+            self.df_regional = pd.DataFrame()
+
         query = "SELECT * FROM regional_metrics_by_year"
-        self.df_regional_by_year = pd.read_sql_query(query, con)
+        try:
+            self.df_regional_by_year = pd.read_sql_query(query, con)
+        except:
+            self.logger.error("Failed to read data from regional_metrics_by_year")
+            self.df_regional_by_year = pd.DataFrame()
 
         con.close()
 
     def clean_data(self):
         """Clean and format the data."""
-        self.df_regional["Country"] = (
-            self.df_regional["Country"].str.replace("_", " ").str.title()
-        )
-        self.df_regional["Model"] = self.df_regional["Model"].str.title()
+        if not self.df_regional.empty:
+            self.df_regional["Country"] = (
+                self.df_regional["Country"].str.replace("_", " ").str.title()
+            )
+            self.df_regional["Model"] = self.df_regional["Model"].str.title()
 
     def plot_heatmap(self):
         """Generate heatmaps of MAPE bins vs. % total area bins."""
@@ -938,7 +1170,6 @@ class RegionalMapper(Geoanalysis):
                     sns.histplot(
                         group_data["Mean Absolute Percentage Error"],
                         label=label,
-                        # clip=(0, None),
                     )
 
                 # Plot a dashed gray line at x=20
@@ -947,10 +1178,23 @@ class RegionalMapper(Geoanalysis):
                 plt.minorticks_on()
                 plt.xlabel("Mean Absolute Percentage Error (%)")
                 plt.ylabel("Frequency")
-                plt.legend(title="Country", title_fontsize="13")
+                plt.legend(title="Country", title_fontsize="16")
+
+                # Adding the title at the top-right corner
+                # plt.text(
+                #     0.95, 0.95,  # Coordinates in axes fraction
+                #     f"Model: {model}",
+                #     transform=plt.gca().transAxes,
+                #     fontsize=14,
+                #     verticalalignment="top",
+                #     horizontalalignment="right",
+                #     bbox=dict(facecolor="white", alpha=0.6, edgecolor="none")
+                # )
 
                 plt.tight_layout()
-                plt.savefig(self.dir_analysis / f"mape_histogram_{model}.png", dpi=250)
+                plt.savefig(
+                    self.dir_analysis / f"histogram_region_{model}_mape.png", dpi=250
+                )
                 plt.close()
 
     def plot_mape_map(self):
@@ -974,13 +1218,14 @@ class RegionalMapper(Geoanalysis):
 
             df_model = df_model.drop(df_tmp.index)
 
-            fname = f"mape_{self.crop}_{df_model['Model'].iloc[0]}.png"
             col = "Mean Absolute Percentage Error"
             countries = df_model["Country"].unique().tolist()
             countries = [country.title().replace("_", " ") for country in countries]
+            crop = df_model["Crop"].unique()[0].title().replace("_", " ")
             df = df_model[df_model["Country"].isin(countries)]
             self.dg = self.dg[self.dg["ADM0_NAME"].isin(countries)]
 
+            fname = f"map_{crop}_{df_model['Model'].iloc[0]}_mape.png"
             plot.plot_df_shpfile(
                 self.dg,
                 df,
@@ -995,7 +1240,7 @@ class RegionalMapper(Geoanalysis):
                 cmap=pal.scientific.sequential.Bamako_20_r,
                 series="sequential",
                 show_bg=False,
-                annotate_regions=False,
+                annotate_regions=self.annotate_regions,
                 loc_legend="lower left",
             )
 
@@ -1019,13 +1264,12 @@ class RegionalMapper(Geoanalysis):
             # Draw a dashed gray line at y=20
             plt.axhline(y=20, color="gray", linestyle="--")
 
-            plt.title("Mean Absolute Percentage Error by Year")
-            plt.xlabel("Year")
+            plt.xlabel("")
             plt.ylabel("Mean Absolute Percentage Error (%)")
             plt.xticks(rotation=0)
 
             plt.tight_layout()
-            plt.savefig(self.dir_analysis / "mape_by_year.png", dpi=250)
+            plt.savefig(self.dir_analysis / "bar_mape_by_year.png", dpi=250)
             plt.close()
 
 
@@ -1058,13 +1302,13 @@ def run(path_config_files=[Path("../config/geocif.txt")]):
 
     df = pd.concat(frames)
 
+    """ For each country, plot yields, conditions, anomalies, etc. """
+    obj.map(df)
+
     """ Map regional error metrics """
     mapper = RegionalMapper(path_config_files, logger, parser)
     mapper.map_regional()
 
-    """ For each country, plot yields, conditions, anomalies, etc. """
-    obj.map(df)
-
 
 if __name__ == "__main__":
     run()
