@@ -332,7 +332,7 @@ def create_output_directory(method, admin_zone, country, crop, path_output):
 
     :return:
     """
-    dir_output = path_output / "fao" / "indices" / method / admin_zone / country / crop
+    dir_output = path_output / "cei" / "indices" / method / admin_zone / country / crop
     os.makedirs(dir_output, exist_ok=True)
 
     return dir_output
@@ -713,3 +713,18 @@ def compute_h_index(values):
             break
 
     return h_index
+
+
+def get_z_value(alpha):
+    """
+    Calculate the z-value for a given alpha level.
+
+    Parameters:
+    alpha (float): The significance level (e.g., 0.05 for a 95% confidence interval)
+
+    Returns:
+    float: The corresponding z-value
+    """
+    from scipy.stats import norm
+
+    return norm.ppf(1 - alpha / 2)
